@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Softwink, Inc. 
-** Copyright (C) 2009-2011 Champ Clark III <champ@softwink.com>
+** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -35,17 +35,19 @@
 #include <pthread.h>
 
 #include "version.h"
+
 #include "sagan.h"
 
-void key_handler(int sigargs )
+void key_handler( _SaganConfig *config )
 {
 
   while(1) {
+
 	int key; 
 
 	key=getchar();
 		   if ( key != 0 ) {
-		   sagan_statistics();
+		   sagan_statistics(config);
 		   }
 
 	}
