@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2011 Softwink, Inc. 
-** Copyright (C) 2009-2011 Champ Clark III <champ@softwink.com>
+** Copyright (C) 2009-2011 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2011 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -48,6 +48,7 @@
 #include <inttypes.h>
 
 #include "sagan.h"
+
 #include "version.h"
 
 
@@ -59,7 +60,6 @@
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
 #include "output-plugins/sagan-snort.h"
-#include "output-plugins/sagan-logzilla.h"
 #endif
 
 #ifdef HAVE_LIBPRELUDE
@@ -74,24 +74,12 @@
 
 #define OVECCOUNT 30
 
-struct _SaganConfig *config;
-struct _SaganDebug *debug;
 struct _SaganCounters *counters;
 
 struct rule_struct *rulestruct;
 struct class_struct *classstruct;
 
 sbool daemonize=0;
-sbool programmode=0;
-sbool dochroot=0;
-
-char saganconf[MAXPATH]=CONFIG_FILE_PATH;
-char *runas=RUNAS;
-
-char sagan_path[MAXPATH];
-
-sbool fifoi=0;
-
 
 /****************************************************************************/
 /* Liblognorm Globals                                                       */
@@ -114,40 +102,36 @@ struct ee_field *field = NULL;
 char *cstr;
 #endif
 
-/* Command line options */
+/* ######################################################################## 
+ * Start of main() thread
+ * ######################################################################## */
+
+int main(int argc, char **argv) {
 
 const struct option long_options[] = {
-	{ "help",         no_argument,          NULL,   'h' },
-	{ "debug", 	  required_argument,	NULL, 	'd' },
-	{ "daemon", 	  no_argument,		NULL,	'D' },
-	{ "program", 	  no_argument,		NULL, 	'p' },
-	{ "user",	  required_argument,	NULL,	'U' },
-	{ "chroot", 	  no_argument, 		NULL, 	'c' },
-	{ "config",	  required_argument, 	NULL, 	'f' },
-	{0, 0, 0, 0}
+        { "help",         no_argument,          NULL,   'h' },
+        { "debug",        required_argument,    NULL,   'd' },
+        { "daemon",       no_argument,          NULL,   'D' },
+        { "user",         required_argument,    NULL,   'U' },
+        { "chroot",       required_argument,    NULL,   'c' },
+        { "config",       required_argument,    NULL,   'f' },
+        { "log",          required_argument,    NULL,   'l' },
+        {0, 0, 0, 0}
 
 };
 
 static const char *short_options =
-"fUd:pDhc";
+"l:f:u:d:c:pDh";
 
 int option_index = 0;
 
-/* ######################################################################## */
-/* Start of main() thread!
- * ######################################################################## */
-
-int main(int argc, char **argv) {
-
 /* Passing Sagan events to output plugins */
 
 struct Sagan_Event *SaganEvent = NULL;
 SaganEvent = malloc(MAX_THREADS * sizeof(struct Sagan_Event));
 
-int endianchk;
-
 /****************************************************************************/
-/* MySQL / PostgreSQL (snort/logzilla) local variables			    */
+/* MySQL / PostgreSQL (snort) local variables			    */
 /****************************************************************************/
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
@@ -160,13 +144,6 @@ pthread_attr_t thread_db_attr;
 pthread_attr_init(&thread_db_attr);
 pthread_attr_setdetachstate(&thread_db_attr,  PTHREAD_CREATE_DETACHED);
 
-pthread_t threadlogzilla_id[MAX_THREADS];
-pthread_attr_t thread_logzilla_attr;
-
-pthread_attr_init(&thread_logzilla_attr);
-pthread_attr_setdetachstate(&thread_logzilla_attr,  PTHREAD_CREATE_DETACHED);
-
-endianchk = checkendian();	// Needed for Snort output
 #endif
 
 /****************************************************************************/
@@ -268,7 +245,6 @@ char *syslog_facility=NULL;
 char syslog_facilitytmp[MAX_MSGSLOT][MAXFACILITY];
 
 char *syslog_priority=NULL;
-//char syslog_prioritytmp[MAXPRIORITY];
 
 char *syslog_level=NULL;
 char syslog_leveltmp[MAX_MSGSLOT][MAXLEVEL];
@@ -311,12 +287,15 @@ char *syspri;
 char *level;
 char *tag;
 char tmpbuf[128];
+memset(tmpbuf,0,128);
 char ipbuf_src[128];
 char ipbuf_dst[128];
 
 char *syslog_msg_case;
 char *s_content_case;
 
+char *runas=RUNAS;
+
 int i;
 int fd=0;
 int b;
@@ -327,13 +306,31 @@ int pcrematch=0;
 time_t t;
 struct tm *now;
 
-memset(&debug, 0, sizeof(debug));
+/* Allocate and clear memory for global structs */
+
+struct _SaganDebug *debug;
 debug = malloc(sizeof(_SaganDebug));
+memset(debug, 0, sizeof(_SaganDebug));
+
+struct _SaganConfig *config;
+config = malloc(sizeof(_SaganConfig));
+memset(config, 0, sizeof(_SaganConfig));
+
+struct _SaganSigArgs *sigargs;
+sigargs = malloc(sizeof(_SaganSigArgs));
+memset(sigargs, 0, sizeof(_SaganSigArgs));
 
-memset(&counters, 0, sizeof(counters));
 counters = malloc(sizeof(_SaganCounters));
+memset(counters, 0, sizeof(_SaganCounters));
 
 
+snprintf(config->sagan_config, sizeof(config->sagan_config), "%s", CONFIG_FILE_PATH);
+
+/* We set the config->sagan_log_filepath to the system default.  It'll be fopen'ed 
+   shortly - 06/03/2011 - Champ Clark III */
+
+snprintf(config->sagan_log_filepath, sizeof(config->sagan_log_filepath), "%s", SAGANLOG);
+
 /* Get command line arg's */
 while ((c = getopt_long(argc, argv, short_options, long_options, &option_index)) != -1) { 
    
@@ -373,23 +370,25 @@ while ((c = getopt_long(argc, argv, short_options, long_options, &option_index))
 	   daemonize=1;
 	   break;
 
-	   case 'U':
+	   case 'u':
 	   runas=optarg;
 	   break;
 
-	   case 'p':
-	   programmode=1;
-	   break;
-
 	   case 'c':
-	   dochroot=1;
+	   sagan_chroot(runas,optarg);
 	   break;
 
 	   case 'f':
-	   strncpy(saganconf,optarg,sizeof(saganconf) - 1);
-	   saganconf[sizeof(saganconf)-1] = '\0';
+	   strncpy(config->sagan_config,optarg,sizeof(config->sagan_config) - 1);		//	strlcpy
+	   config->sagan_config[sizeof(config->sagan_config)-1] = '\0';
 	   break;
 
+	   case 'l':
+	   strncpy(config->sagan_log_filepath,optarg,sizeof(config->sagan_log_filepath) - 1);
+	   config->sagan_log_filepath[sizeof(config->sagan_log_filepath)-1] = '\0';
+	   break;
+	   
+
 	  default:
           fprintf(stderr, "Invalid argument! See below for command line switches.\n");
           sagan_usage();
@@ -398,11 +397,16 @@ while ((c = getopt_long(argc, argv, short_options, long_options, &option_index))
   	  }
 }
 
-/* create the signal handling thread */
+/* Open the sagan.log file.  Moved from sagan-config.c as it became to complex 
+   06/03/2011 - Champ Clark */
 
-sig_thread_args[0].daemonize = daemonize;
+if ((config->sagan_log_stream = fopen(config->sagan_log_filepath, "a")) == NULL) {
+    fprintf(stderr, "[E] [%s, line %d] Cannot open %s!\n", __FILE__, __LINE__, config->sagan_log_filepath);
+    exit(1);
+    }
 
-load_config();
+
+load_config( debug, config );
 
 /* Load/init liblognorm definitions.  I tried to move this into a subroutine,
  * but that ended up causing segfaults on ln_normalize() or causing 
@@ -412,82 +416,81 @@ load_config();
  */
 
 #ifdef HAVE_LIBLOGNORM
-if((ctx = ln_initCtx()) == NULL) sagan_log(1, "[%s, line %d] Cannot initialize liblognorm context.", __FILE__, __LINE__);
-if((eectx = ee_initCtx()) == NULL) sagan_log(1, "[%s, line %d] Cannot initialize libee context.", __FILE__, __LINE__);
+if((ctx = ln_initCtx()) == NULL) sagan_log(config, 1, "[%s, line %d] Cannot initialize liblognorm context.", __FILE__, __LINE__);
+if((eectx = ee_initCtx()) == NULL) sagan_log(config, 1, "[%s, line %d] Cannot initialize libee context.", __FILE__, __LINE__);
 
 ln_setEECtx(ctx, eectx);
 
 for (i=0; i < counters->liblognormtoload_count; i++) { 
-sagan_log(0, "Loading %s for normalization.", liblognormtoloadstruct[i].filepath);
-if (stat(liblognormtoloadstruct[i].filepath, &fileinfo)) sagan_log(1, "%s was not fonnd.", liblognormtoloadstruct[i].filepath);
+sagan_log(config, 0, "Loading %s for normalization.", liblognormtoloadstruct[i].filepath);
+if (stat(liblognormtoloadstruct[i].filepath, &fileinfo)) sagan_log(config, 1, "%s was not fonnd.", liblognormtoloadstruct[i].filepath);
 ln_loadSamples(ctx, liblognormtoloadstruct[i].filepath);
 }
 #endif
 
-sagan_log(0, "Configuration file %s loaded and %d rules loaded.", saganconf, counters->rulecount);
-sagan_log(0, "Sagan version %s is firing up!", VERSION);
+sagan_log(config, 0, "Configuration file %s loaded and %d rules loaded.", config->sagan_config, counters->rulecount);
+sagan_log(config, 0, "Sagan version %s is firing up!", VERSION);
+
+/* We go ahead and assign values to SaganSigArgs (struct sig_thread_args).  This
+ * struct is always used by the sig_handler thread,  and sometimes used by the
+ * plog_handler (below).  So we assign values now */
+
+sigargs->daemonize = daemonize;
+sigargs->debug     = debug;
+sigargs->config    = config;
 
 #ifdef HAVE_LIBPCAP
 
 /* Spawn a thread to 'sniff' syslog traffic (sagan-plog.c).  This redirects syslog
- * traffic to the /dev/log socket */
+   traffic to the /dev/log socket.  This needs "root" access,  so we drop priv's
+   after this thread is started */
 
 if ( config->plog_flag ) { 
 
-if ( pthread_create( &pcap_thread, NULL, (void *)plog_handler, NULL)) {
-        removelockfile();
-        sagan_log(1, "[%s, line %d] Error creating libpcap handler thread.", __FILE__, __LINE__);
+if ( pthread_create( &pcap_thread, NULL, (void *)plog_handler, sigargs )) {
+
+        removelockfile(config);
+        sagan_log(config, 1, "[%s, line %d] Error creating libpcap handler thread.", __FILE__, __LINE__);
         }
+
+sleep(1); 	/* Sleep to avoid race between main() and plog thread 
+		   plog thread needs "root" rights before sagan_droppriv().
+		   In some cases main() run sagan_droppriv() before thread
+		   can complete - Champ Clark - 07/20/2011 */
+			
 }
 #endif
 
-droppriv(runas, config->sagan_fifo);		/* Become the Sagan user */
-sagan_log(0, "---------------------------------------------------------------------------");
-
-/* Create signal handler thread */ 
-
-if (daemonize == 0) { 
-if ( pthread_create( &sig_thread, NULL, (void *)sig_handler, &sig_thread_args )) {
-        removelockfile();
-        sagan_log(1, "[%s, line %d] Error creating signal handler thread.", __FILE__, __LINE__);
-        }
-}
+sagan_droppriv(config, runas);		/* Become the Sagan user */
+sagan_log(config, 0, "---------------------------------------------------------------------------");
 
 /* Open sagan alert file */
 
 if (( config->sagan_alert_stream = fopen(config->sagan_alert_filepath, "a" )) == NULL ) {
-removelockfile();
-sagan_log(1, "[%s, line %d] Can't open %s!", __FILE__, __LINE__, config->sagan_alert_filepath);
+removelockfile(config);
+sagan_log(config, 1, "[%s, line %d] Can't open %s!", __FILE__, __LINE__, config->sagan_alert_filepath);
 }
 
-if ( config->sagan_ext_flag ) sagan_log(0, "Max external threads : %d", config->max_external_threads);
+if ( config->sagan_ext_flag ) sagan_log(config, 0, "Max external threads : %d", config->max_external_threads);
 
 #ifdef HAVE_LIBESMTP
-if ( config->sagan_esmtp_flag ) sagan_log(0, "Max SMTP threads     : %d", config->max_email_threads);
+if ( config->sagan_esmtp_flag ) sagan_log(config, 0, "Max SMTP threads     : %d", config->max_email_threads);
 #endif
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-if ( config->logzilla_dbtype ) { 
-sagan_log(0, "Max Logzilla threads : %d", config->max_logzilla_threads);
-logzilla_db_connect();
-}
-#endif 
-
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-
-sig_thread_args[0].daemonize = daemonize;
 
 if ( config->dbtype ) { 
 
-sagan_log(0, "Max database threads : %d", config->maxdb_threads);
+sagan_log(config, 0, "Max database threads : %d", config->maxdb_threads);
 
-db_connect();
-get_sensor_id( config->sagan_hostname, config->sagan_interface, config->sagan_filter, config->sagan_detail, config->dbtype);
-sagan_log(0, "Sensor ID            : %d", config->sensor_id);
-cid = get_cid( config->sensor_id, config->dbtype );
+db_connect(config);
+
+get_sensor_id( debug, config ); 
+sagan_log(config, 0, "Sensor ID            : %d", config->sensor_id);
+cid = get_cid( debug, config );
 cid++;
 counters->sigcid=cid;
-sagan_log(0, "Next CID             : %" PRIu64 "", cid);
+sagan_log(config, 0, "Next CID             : %" PRIu64 "", cid);
 
 }
 #endif
@@ -496,11 +499,11 @@ sagan_log(0, "Next CID             : %" PRIu64 "", cid);
 
 if ( config->sagan_prelude_flag ) {
 
-sagan_log(0, "Prelude profile: %s", config->sagan_prelude_profile);
-sagan_log(0, "Max Prelude threads: %d", config->max_prelude_threads);
-sagan_log(0, "");  /* libprelude dumps some information.  This is to make it pretty */
+sagan_log(config, 0, "Prelude profile: %s", config->sagan_prelude_profile);
+sagan_log(config, 0, "Max Prelude threads: %d", config->max_prelude_threads);
+sagan_log(config, 0, "");  /* libprelude dumps some information.  This is to make it pretty */
 
-PreludeInit();
+PreludeInit(config);
 }
 
 #endif
@@ -509,59 +512,32 @@ PreludeInit();
 
 if ( config->sagan_unified2_flag ) { 
 
-sagan_log(0, "");
-sagan_log(0, "Unified2 file: %s", config->unified2_filepath);
-sagan_log(0, "Unified2 limit: %dM", config->unified2_limit  / 1024 / 1024 );
-Unified2InitFile( );
+sagan_log(config, 0, "");
+sagan_log(config, 0, "Unified2 file: %s", config->unified2_filepath);
+sagan_log(config, 0, "Unified2 limit: %dM", config->unified2_limit  / 1024 / 1024 );
+Unified2InitFile( config );
 
 }
 
 #endif
 
 
-sagan_log(0, "");
+sagan_log(config, 0, "");
 
-if ( fifoi == 0 ) { 
-   if ( programmode == 0 ) 
-      { 
-      sagan_log(0, "No FIFO option found,  assuming syslog-ng 'program' mode.");
-      programmode = 1;
-      }
-   } else { 
-   sagan_log(0, "Opening syslog FIFO (%s)", config->sagan_fifo);
-   fd = open(config->sagan_fifo, O_RDONLY); 
-   }
-
-
-sagan_log(0, "");
-sagan_log(0, " ,-._,-. 	-*> Sagan! <*-");
-sagan_log(0, " \\/)\"(\\/	Version %s", VERSION);
-sagan_log(0, "  (_o_)  	By Champ Clark III & The Softwink Team: http://www.softwink.com");
-sagan_log(0, "  /   \\/)	Copyright (C) 2009-2011 Softwink, Inc., et al.");
-sagan_log(0, " (|| ||) 	Using PCRE version: %s", pcre_version());
-sagan_log(0, "  oo-oo     Sagan is processing events.....");
-sagan_log(0, "");
+sagan_log(config, 0, "");
+sagan_log(config, 0, " ,-._,-. 	-*> Sagan! <*-");
+sagan_log(config, 0, " \\/)\"(\\/	Version %s", VERSION);
+sagan_log(config, 0, "  (_o_)	Champ Clark III & The Quadrant InfoSec Team [quadrantsec.com]");
+sagan_log(config, 0, "  /   \\/)	Copyright (C) 2009-2011 Quadrant Information Security, et al.");
+sagan_log(config, 0, " (|| ||) 	Using PCRE version: %s", pcre_version());
+sagan_log(config, 0, "  oo-oo     Sagan is processing events.....");
+sagan_log(config, 0, "");
 
 /* Become a daemon if requested */
 
 if ( daemonize )
 {
-
-/* Unblock signals so the daemon can catch them */
-
-sigfillset(&signal_set);
-
-pthread_sigmask(SIG_UNBLOCK, &signal_set, NULL );
-
-signal (SIGHUP,  &sig_handler_daemon );
-signal (SIGINT,  &sig_handler_daemon );
-signal (SIGQUIT, &sig_handler_daemon );
-signal (SIGTERM, &sig_handler_daemon );
-signal (SIGABRT, &sig_handler_daemon );
-signal (SIGSEGV, &sig_handler_daemon );
-signal (SIGUSR1, &sig_handler_daemon );
-
-sagan_log(0, "Becoming a daemon!");
+sagan_log(config, 0, "Becoming a daemon!");
 
 pid_t pid = 0;
 setsid();
@@ -569,44 +545,58 @@ pid = fork();
 if (pid == 0) {} else { exit(0); }
 } 
 
-/* We don't want the key_handler() if we're in program/daemon mode! */
+/* Create the signal handlers thread _after_ the fork() so it can properly 
+ * handly signals - Champ Clark III - 06/13/2011 */
 
-if (!daemonize && !programmode) { 
+if ( pthread_create( &sig_thread, NULL, (void *)sig_handler, sigargs )) {
+        removelockfile(config);
+        sagan_log(config, 1, "[%s, line %d] Error creating signal handler thread.", __FILE__, __LINE__);
+        }
 
-if (pthread_create( &key_thread, NULL, (void *)key_handler, NULL )) { ;
-	removelockfile();
-	sagan_log(1, "[%s, line %d] Error creating key_handler thread.", __FILE__, __LINE__);
+
+/* We don't want the key_handler() if we're in daemon mode! */
+
+if (!daemonize) { 
+
+if (pthread_create( &key_thread, NULL, (void *)key_handler, config )) { ;
+	removelockfile(config);
+	sagan_log(config, 1, "[%s, line %d] Error creating key_handler thread.", __FILE__, __LINE__);
 	}
 
 }
 
+/* We do this after forking so init scripts can complete */
 
 /* Check lock file _after_ thread.  If you don't it'll retreive the wrong pid
  * and incorrectly believe there is a stale lock file if --daemon */
 
-checklockfile();
+checklockfile(config);
+
+sagan_log(config, 0, "Attempting to open syslog FIFO (%s).", config->sagan_fifo);
+
+if ( fd == 0 ) fd = open(config->sagan_fifo, O_RDONLY);
+
+sagan_log(config, 0, "Successfully opened FIFO (%s).", config->sagan_fifo);
 
 while(1) { 
 
-		if ( fifoi == 1 ) { 
-
                 if(fd < 0) {
-		        removelockfile();
-			sagan_log(1, "[%s, line %d] Error opening in FIFO! %s (Errno: %d)", __FILE__, __LINE__, config->sagan_fifo, errno);
-               }
+		        removelockfile(config);
+			sagan_log(config, 1, "[%s, line %d] Error opening in FIFO! %s (Errno: %d)", __FILE__, __LINE__, config->sagan_fifo, errno);
+                        }
 
                 i = read(fd, &c, 1);
                 
-		if(i < 0) {
-  	               removelockfile();
-                       sagan_log(1, "[%s, line %d] Error reading FIFO! %s (Errno: %d)", __FILE__, __LINE__, config->sagan_fifo, errno);
+		 if(i < 0) {
+  	                removelockfile(config);
+                        sagan_log(config, 1, "[%s, line %d] Error reading FIFO! %s (Errno: %d)", __FILE__, __LINE__, config->sagan_fifo, errno);
                         }
 
 		/* Error on reading (FIFO writer left) and we have no 
 		 * previous error state. */
 
 		if (i == 0 && fifoerr == 0 ) { 
-		   sagan_log(0, "FIFO closed (writer exited). Will start processing when writer resumes.");
+		   sagan_log(config, 0, "FIFO closed (writer exited). Will start processing when writer resumes.");
 		   fifoerr=1;
 		   }
 
@@ -614,7 +604,7 @@ while(1) {
 		 * then the write is back online. */
 
 		if ( fifoerr == 1 && i == 1 )  { 
-		   sagan_log(0,"FIFO writer detected, resuming...");
+		   sagan_log(config, 0,"FIFO writer detected, resuming...");
 		   fifoerr=0;  /* Rest error state */
 		   }
 
@@ -626,15 +616,7 @@ while(1) {
                 snprintf(syslogtmp, sizeof(syslogtmp), "%c", c);
                 strncat(syslogstring, syslogtmp, 1); 
 
-		} else { 
-
-
-		if (!fgets(syslogstring, sizeof(syslogstring), stdin)) { 
-		   sagan_log(0, "Dropped input in 'program' mode!");
-		}
-		}
-
-		if ( c == '\n' || c == '\r' || fifoi == 0  ) 
+		if ( c == '\n' || c == '\r' ) 
                 {
 
 		counters->sagantotal++;
@@ -653,66 +635,67 @@ while(1) {
 		
 		if (syslog_host == NULL ) { 
 		   syslog_host = "SAGAN: HOST ERROR"; 
-		   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'host'");
+		   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'host'");
 		   }
 		
 		syslog_facility=strtok_r(NULL, "|", &tok);
 		if ( syslog_facility == NULL ) { 
 		   syslog_facility = "SAGAN: FACILITY ERROR";
-		   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'facility'");
+		   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'facility'");
 		   }
 
                 syslog_priority=strtok_r(NULL, "|", &tok);
 		if ( syslog_priority == NULL ) { 
 		   syslog_priority = "SAGAN: PRIORITY ERROR";
-		   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'priority'");
+		   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'priority'");
 		   }
 
                 syslog_level=strtok_r(NULL, "|", &tok);
 		if ( syslog_level == NULL ) { 
 		   syslog_level = "SAGAN: LEVEL ERROR";
-		   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'priority'");
+		   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'priority'");
 		   }
 
                 syslog_tag=strtok_r(NULL, "|", &tok);
                 if ( syslog_tag == NULL ) {
                    syslog_tag = "SAGAN: TAG ERROR";
-                   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'tag'");
+                   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'tag'");
                    }
 
                 syslog_date=strtok_r(NULL, "|", &tok);
                 if ( syslog_date == NULL ) {
                    syslog_date = "SAGAN: DATE ERROR";
-                   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'date'");
+                   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'date'");
                    }
 
                 syslog_time=strtok_r(NULL, "|", &tok);
                 if ( syslog_time == NULL ) {
                    syslog_time = "SAGAN: TIME ERROR";
-                   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'time'");
+                   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'time'");
                    }
 
                 syslog_program=strtok_r(NULL, "|", &tok);
                 if ( syslog_program == NULL ) {
                    syslog_program = "SAGAN: PROGRAM ERROR";
-                   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'program'");
+                   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'program'");
                    } else {
                    syslog_msg=syslog_program + strlen(syslog_program) + 1;
                    }
 
                 if ( syslog_msg == NULL ) {
                    syslog_msg = "SAGAN: MESSAGE ERROR";
-                   if ( !fifoerr ) sagan_log(0, "Sagan received a malformed 'message'\n");
+                   if ( !fifoerr ) sagan_log(config, 0, "Sagan received a malformed 'message'\n");
                    }
 
 
-if (debug->debugsyslog) sagan_log(0, "%s|%s|%s|%s|%s|%s|%s|%s|%s", syslog_host, syslog_facility, syslog_priority, syslog_level, syslog_tag, syslog_date, syslog_time, syslog_program, syslog_msg);
+               /* Strip any \n or \r from the syslog_msg */
 
-/* If in "program" mode,  we need the \r \n's */
+               syslog_msg[strcspn ( syslog_msg, "\n" )] = '\0';
+               syslog_msg[strcspn ( syslog_msg, "\r" )] = '\0';
+
+
+if (debug->debugsyslog) sagan_log(config, 0, "%s|%s|%s|%s|%s|%s|%s|%s|%s", syslog_host, syslog_facility, syslog_priority, syslog_level, syslog_tag, syslog_date, syslog_time, syslog_program, syslog_msg);
 
-if ( programmode == 0 ) {
-   syslog_msg[strcspn ( syslog_msg, "\n" )] = '\0';
-   }
 
 		/* Search for matches */
 
@@ -791,11 +774,11 @@ if ( programmode == 0 ) {
 		      s_content_case=rulestruct[b].s_content[z];
 		      toupperc(syslog_msg_case);
 		      toupperc(s_content_case); 
-		      if (strstr(syslog_msg_case, s_content_case )) pcrematch++;   // rc=1;
+		      if (strstr(syslog_msg_case, s_content_case )) pcrematch++;
 		      } else { 
 
 		   /* If case sensitive */
-		   if (strstr(syslog_msg, rulestruct[b].s_content[z] )) pcrematch++;  // rc=1;
+		   if (strstr(syslog_msg, rulestruct[b].s_content[z] )) pcrematch++;
 		   }
 		  }
 		 }
@@ -845,7 +828,7 @@ if ( programmode == 0 ) {
                         ee_fmtEventToRFC5424(lnevent, &str);
                         cstr = es_str2cstr(str, NULL);
 			
-			if ( debug->debugnormalize ) sagan_log(0, "Normalize output: %s", cstr);
+			if ( debug->debugnormalize ) sagan_log(config, 0, "Normalize output: %s", cstr);
 
 			propName = es_newStrFromBuf("src-ip", 6);
 			if((field = ee_getEventField(lnevent, propName)) != NULL) {
@@ -888,14 +871,14 @@ if ( programmode == 0 ) {
                         propName = es_newStrFromBuf("src-host", 8);
                         if((field = ee_getEventField(lnevent, propName)) != NULL) {
                            str = ee_getFieldValueAsStr(field, 0);
-			   snprintf(ipbuf_src, sizeof(ipbuf_src), "%s", dns_lookup(es_str2cstr(str, NULL)));
+			   snprintf(ipbuf_src, sizeof(ipbuf_src), "%s", dns_lookup(config, es_str2cstr(str, NULL)));
 			   ip_src=ipbuf_src;
                            }
 
                        propName = es_newStrFromBuf("dst-host", 8);
                         if((field = ee_getEventField(lnevent, propName)) != NULL) {
                            str = ee_getFieldValueAsStr(field, 0);
-			   snprintf(ipbuf_dst, sizeof(ipbuf_dst), "%s", dns_lookup(es_str2cstr(str, NULL)));
+			   snprintf(ipbuf_dst, sizeof(ipbuf_dst), "%s", dns_lookup(config, es_str2cstr(str, NULL)));
 			   ip_dst=ipbuf_dst;
 
                            }
@@ -928,7 +911,7 @@ if ( rulestruct[b].normalize == 0 ) {
  }
 
 if ( rulestruct[b].s_find_port == 1 ) {
-   src_port = parse_port_simple(syslog_msg);
+   src_port = parse_port_simple(config, syslog_msg);
     } else {
    src_port = config->sagan_port;
    }
@@ -992,7 +975,7 @@ if ( rulestruct[b].threshold_type != 0 ) {
 		if ( rulestruct[b].threshold_count < threshbysrc[i].count ) 
 			{ 
 			thresh_log_flag = 1;
-			sagan_log(0, "Threshold SID %s by source IP address. [%s]", threshbysrc[i].sid, ip_src);
+			sagan_log(config, 0, "Threshold SID %s by source IP address. [%s]", threshbysrc[i].sid, ip_src);
 			counters->threshold_total++;
 			}
   			
@@ -1032,7 +1015,7 @@ if ( rulestruct[b].threshold_type != 0 ) {
 
 	if ( rulestruct[b].threshold_count < threshbydst[i].count ) {
 	   thresh_log_flag = 1;
-	   sagan_log(0, "Threshold SID %s by source IP address. [%s]", threshbysrc[i].sid, ip_dst);
+	   sagan_log(config, 0, "Threshold SID %s by source IP address. [%s]", threshbysrc[i].sid, ip_dst);
 	   counters->threshold_total++;
 	   }
          }
@@ -1092,7 +1075,6 @@ SaganEvent[threadid].src_port  =       src_porttmp[msgslot];
 SaganEvent[threadid].found     =       b;
 SaganEvent[threadid].program   =       syslog_programtmp[msgslot];
 SaganEvent[threadid].message   =       sysmsg[msgslot];
-SaganEvent[threadid].endian    =       endianchk;
 SaganEvent[threadid].time      =       syslog_timetmp[msgslot];
 SaganEvent[threadid].date      =       syslog_datetmp[msgslot];
 SaganEvent[threadid].f_msg     =       s_msgtmp[msgslot]; 
@@ -1102,6 +1084,9 @@ SaganEvent[threadid].tag       =       syslog_tagtmp[msgslot];
 SaganEvent[threadid].host      =       syslog_hosttmp[msgslot];
 SaganEvent[threadid].event_time_sec = 	time(NULL);
 
+SaganEvent[threadid].debug     = 	debug;
+SaganEvent[threadid].config    = 	config;
+
 }
 
 
@@ -1137,13 +1122,13 @@ if ( counters->threadpreludec < config->max_prelude_threads ) {
 	if ( counters->threadpreludec > counters->threadmaxpreludec ) counters->threadmaxpreludec=counters->threadpreludec;
 
 	if ( pthread_create ( &threadprelude_id[threadid], &thread_prelude_attr, (void *)sagan_prelude, &SaganEvent[threadid] ) ) { 
-		removelockfile();
-	        sagan_log(1, "[%s, line %d] Error creating Prelude thread", __FILE__, __LINE__);
+		removelockfile(config);
+	        sagan_log(config, 1, "[%s, line %d] Error creating Prelude thread", __FILE__, __LINE__);
 	        } 
 	      	 } else { 
                 counters->sagandrop++;
                 counters->saganpreludedrop++;
-                sagan_log(0, "Prelude thread call handler: Out of threads\n");
+                sagan_log(config, 0, "Prelude thread call handler: Out of threads\n");
               	}
 }
 #endif
@@ -1174,14 +1159,14 @@ if ( config->sagan_esmtp_flag == 1 && thresh_log_flag == 0 ) {
 		    if ( counters->threademailc > counters->threadmaxemailc ) counters->threadmaxemailc=counters->threademailc;
 
                     if ( pthread_create( &threademail_id[threadid], &thread_email_attr, (void *)sagan_esmtp_thread, &SaganEvent[threadid] ) ) {
-		       removelockfile();
-                       sagan_log(1, "[%s, line %d] Error creating SMTP thread", __FILE__, __LINE__);
+		       removelockfile(config);
+                       sagan_log(config, 1, "[%s, line %d] Error creating SMTP thread", __FILE__, __LINE__);
                        }
 
 			} else { 
 		       counters->sagandrop++;
 		       counters->saganesmtpdrop++;
-		       sagan_log(0, "SMTP thread call handler: Out of threads\n");
+		       sagan_log(config, 0, "SMTP thread call handler: Out of threads\n");
           }
       }
    }
@@ -1201,44 +1186,17 @@ if ( config->sagan_ext_flag == 1 && thresh_log_flag == 0 ) {
 	if ( counters->threadextc > counters->threadmaxextc ) counters->threadmaxextc=counters->threadextc;
 	
 		if ( pthread_create( &threadext_id[threadid], &thread_ext_attr, (void *)sagan_ext_thread, &SaganEvent[threadid] ) ) { 
-		     removelockfile();
-		     sagan_log(1, "[%s, line %d] Error creating external call thread", __FILE__, __LINE__);
+		     removelockfile(config);
+		     sagan_log(config, 1, "[%s, line %d] Error creating external call thread", __FILE__, __LINE__);
 		     }
 		      } else {
 		     counters->saganexternaldrop++;
 		     counters->sagandrop++; 
-		     sagan_log(0, "External thread call handler: Out of threads\n");
+		     sagan_log(config, 0, "External thread call handler: Out of threads\n");
 		   }
 }
 
 
-/****************************************************************************/
-/* Logzilla,  alert only,  thread call                                      */
-/****************************************************************************/
-
-#if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
-
-if ( config->logzilla_dbtype != 0 && thresh_log_flag == 0 ) { 
-		   
-	if ( counters->threadlogzillac < config->max_logzilla_threads) { 
-		      
-	        counters->threadlogzillac++;
-		      
-		if ( counters->threadlogzillac > counters->threadmaxlogzillac ) counters->threadmaxlogzillac=counters->threadlogzillac;
-
-                     if ( pthread_create( &threadlogzilla_id[threadid], &thread_logzilla_attr, (void *)sagan_logzilla_thread, &SaganEvent[threadid]) ) {
-                          removelockfile();
-                          sagan_log(1, "[%s, line %d] Error creating database thread.", __FILE__, __LINE__);
-		        }
-		           } else { 
-		          counters->saganlogzilladrop++;
-		          counters->sagandrop++;
-		          sagan_log(0, "Logzilla thread handler: Out of threads");
-		  }
-}
-
-#endif
-
 /****************************************************************************/
 /* Snort database thread call                                               */
 /****************************************************************************/
@@ -1246,6 +1204,8 @@ if ( config->logzilla_dbtype != 0 && thresh_log_flag == 0 ) {
 
 #if defined(HAVE_LIBMYSQLCLIENT_R) || defined(HAVE_LIBPQ)
 
+config->endian = checkendian();    // Needed for Snort output
+
 if ( config->dbtype != 0 && thresh_log_flag == 0 ) { 
 
         counters->threaddbc++;
@@ -1260,13 +1220,13 @@ if ( config->dbtype != 0 && thresh_log_flag == 0 ) {
 				SaganEvent[threadid].cid = cid;
 
 				if ( pthread_create( &threaddb_id[threadid], &thread_db_attr, (void *)sagan_db_thread, &SaganEvent[threadid]) ) { 
-		    		   removelockfile();
-		    		   sagan_log(1, "[%s, line %d] Error creating database thread.", __FILE__, __LINE__);
+		    		   removelockfile(config);
+		    		   sagan_log(config, 1, "[%s, line %d] Error creating database thread.", __FILE__, __LINE__);
 		    		   }
 		    		    } else { 
 		    		   counters->sagansnortdrop++;
 		    		   counters->sagandrop++;
-		    		   sagan_log(0, "Snort database thread handler: Out of threads");
+		    		   sagan_log(config, 0, "Snort database thread handler: Out of threads");
 	        		   }
 }
 #endif
@@ -1284,6 +1244,7 @@ syslogtmp[0]='\0';
 }
 
 } /* End of while(1) */
+
 } /* End of main */
 
 
